unit UPath;

{ EOFile by YT }

interface

uses
	Windows, ShlObj, ActiveX, SysUtils,
	SysUtilsW, WinTips, ShlTips,
	EOFileIntf;

function ParsePath(const S: WideString): TPath;
function ComparePath(const P1, P2: TPath): Boolean;
function CopyPath(const P: TPath): TPath;
procedure InitializePath(var P: TPath);
procedure FinalizePath(var P: TPath);

implementation

{$IFDEF PATH_AS_INTERFACE}
type
	IPath = interface(IInterface)
		['{2C3B3CB2-8FF1-46F1-8E80-14E27EDB851B}']
		function GetItemIDList: PItemIDList;
		function GetDirectory: WideString;
		function GetArchive: WideString;
		property ItemIDList: PItemIDList read GetItemIDList;
		property Directory: WideString read GetDirectory;
		property Archive: WideString read GetArchive;
	end;

type
	PPathImpl = ^TPathImpl;
	PPathVMT = ^TPathVMT;
	TPathVMT = record
		QueryInterface: function(Self: PPathImpl; const IID: TGUID; out Obj): HResult; stdcall;
		AddRef: function(Self: PPathImpl): Integer; stdcall;
		Release: function(Self: PPathImpl): Integer; stdcall;
		GetItemIDList: function(Self: PPathImpl): PItemIDList;
		GetDirectory: function(Self: PPathImpl): WideString;
		GetArchive: function(Self: PPathImpl): WideString;
	end;
	TPathImpl = record
		VMT: PPathVMT;
		RefCount: Integer;
		ItemIDList: PItemIDList;
		Directory: WideString;
		Archive: WideString;
	end;

function Path_QueryInterface(Self: PPathImpl; const IID: TGUID; out Obj): HResult; stdcall;
begin
	Result := E_NOINTERFACE
end;

function Path_AddRef(Self: PPathImpl): Integer; stdcall;
begin
	Result := InterlockedIncrement(Self^.RefCount)
end;

function Path_Release(Self: PPathImpl): Integer; stdcall;
begin
	Result := InterlockedDecrement(Self^.RefCount);
	if Result = 0 then
	begin
		if Self^.ItemIDList <> nil then CoTaskMemFree(Self^.ItemIDList);
		Dispose(Self);
	end;
end;

function Path_GetItemIDList(Self: PPathImpl): PItemIDList;
begin
	Result := Self^.ItemIDList
end;

function Path_GetDirectory(Self: PPathImpl): WideString;
begin
	Result := Self^.Directory
end;

function Path_GetArchive(Self: PPathImpl): WideString;
begin
	Result := Self^.Archive
end;

const
	PathVMT: TPathVMT = (
		QueryInterface: Path_QueryInterface;
		AddRef:         Path_AddRef;
		Release:        Path_Release;
		GetItemIDList:  Path_GetItemIDList;
		GetDirectory:   Path_GetDirectory;
		GetArchive:     Path_GetArchive;
	);

function MakePath(ItemIDList: PItemIDList; const Dir, Arc: WideString): TPath;
var
	Self: PPathImpl;
begin
	New(Self);
	Self^.VMT := @PathVMT;
	Self^.RefCount := 1;
	Self^.ItemIDList := ItemIDList;
	Self^.Directory := Dir;
	Self^.Archive := Arc;
	PPathImpl(Result) := Self;
end;

function ParsePath(const S: WideString): IPath;
var
	Desktop: IShellFolder;
	E, A: DWORD;
	ItemIDList: PItemIDList;
	Dir, Arc: WideString;
begin
	ItemIDList := nil;
	Dir := '';
	Arc := '';

	if S = '' then
	begin
		SHGetSpecialFolderLocation(0, CSIDL_DESKTOP, ItemIDList);
	end
	else if FileSystemExists(S) then
	begin
		Dir := ExtractLongPathName(ExpandFileName(S));
	end
	else
	begin
		SHGetDesktopFolder(Desktop);
		if Desktop.ParseDisplayName(0, nil, Pointer(S), E, ItemIDList, A) = S_OK then
			Dir := S
		else
			Dir := GetCurrentDir; {ftH}
	end;

	Result := MakePath(ItemIDList, Dir, Arc)
end;

{$ELSE}

function ParsePath(const S: WideString): TPath;
var
	Desktop: IShellFolder;
	E, A: DWORD;
	P: PWideChar;
	Pos: Integer;
	S2: WideString;
begin
	InitializePath(Result);
	S2 := S;
	P := WideStrScan(Pointer(S2), '/');
	if P <> nil then
	begin
		Pos := P - PWideChar(Pointer(S2)) + 1;
		SetString(Result.Archive, P + 1, Length(S2) - Pos);
		Delete(S2, Pos, MaxInt);
	end;
	if S2 = '' then
	begin
		SHGetSpecialFolderLocation(0, CSIDL_DESKTOP, Result.ItemIDList);
	end
	else if FileSystemExists(S2) then
	begin
		Result.Directory := ExtractLongPathName(ExpandFileName(S2));
	end
	else
	begin
		SHGetDesktopFolder(Desktop);
		if Desktop.ParseDisplayName(0, nil, Pointer(S2), E, Result.ItemIDList, A) = S_OK then
			Result.Directory := S2
		else
			Result.Directory := GetCurrentDir; {ftH}
	end;
end;

function ComparePath(const P1, P2: TPath): Boolean;
var
	Size: Integer;
begin
	if P1.Extension <> P2.Extension then
		Result := False
	else if (P1.ItemIDList <> nil) <> (P2.ItemIDList <> nil) then
		Result := False
	else if (P1.ItemIDList = nil) and (P2.ItemIDList = nil) then
		Result := (CompareFileName(P1.Directory, P2.Directory) = 0) and
						(CompareFileName(P1.Archive, P2.Archive) = 0)
	else
	begin
		Size := GetItemIDListSize(P1.ItemIDList);
		Result := (Size = GetItemIDListSize(P2.ItemIDList)) and
						CompareMem(P1.ItemIDList, P2.ItemIDList, Size)
	end
end;

function CopyPath(const P: TPath): TPath;
begin
	Result := P;
	Result.ItemIDList := CopyItemIDList(P.ItemIDList);
end;

procedure InitializePath(var P: TPath);
begin
	P.ItemIDList := nil;
	P.Directory := '';
	P.Archive := '';
	P.Extension := nil;
end;

procedure FinalizePath(var P: TPath);
begin
	if P.ItemIDList <> nil then
	begin
		CoTaskMemFree(P.ItemIDList);
		P.ItemIDList := nil
	end;
end;

{$ENDIF}

end.
